%
%
%  Find Spots Close to Filaments for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Spots Functions">
%        <Item name="Find Spots Near to Filament Border" icon="Matlab" tooltip="Find spot center close to filament border.">
%          <Command>MatlabXT::XT_AT_SpotsCloseToFilaments(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpSpots">
%          <Item name="Find Spots Near to Filament Border" icon="Matlab" tooltip="Find spot center close to filament border.">
%            <Command>MatlabXT::XT_AT_SpotsCloseToFilaments(%i)</Command>
%          </Item>
%        </SurpassComponent>
%        <SurpassComponent name="bpFilaments">
%          <Item name="Find Spots Near to Filament Border" icon="Matlab" tooltip="Find spot center close to filament border.">
%            <Command>MatlabXT::XT_AT_SpotsCloseToFilaments(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
%
%  
%  Description:
%
%   The user has to create a Spots and a Filaments component. 
%   This XTension segments the spots into two groups: Spots which are 
%       near to the filamentous structure will be colored red, the others 
%       blue. 
%   This might help to count spines.
%
%   Usage Example:
%       - Load "C2CZOSmall.ims".
%       - Create Spots (ch1, 0.5um, on, 0.5um, thresh.10)
%       - Create Filaments (default values, ratio branch / trunk = 3)
%       - Select tools tab from spots component
%       - Start "Find Spots near to a Filaments"
%       - Ratio = 0.7
%
%

function XT_AT_SpotsCloseToFilaments(aImarisApplicationID)

if isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    vImarisApplication = aImarisApplicationID;
else
    % connect to Imaris interface
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
end

%
% GUI - user interactions
%

% the user has to create a scene with a filament and some spots
if isequal(vImarisApplication.GetSurpassScene, [])
    msgbox('Please create a Filaments and some Spots!');
    return;
end

% THE filament and THE spot object
vFilaments = vImarisApplication.GetFactory.ToFilaments(vImarisApplication.GetSurpassSelection);
vSpots = vImarisApplication.GetFactory.ToSpots(vImarisApplication.GetSurpassSelection);

% find a filament and some spots from the surpass scene
vSurpassScene = vImarisApplication.GetSurpassScene;
for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
    vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
    if isequal(vSpots, [])
        if vImarisApplication.GetFactory.IsSpots(vDataItem)
            vSpots = vImarisApplication.GetFactory.ToSpots(vDataItem);
        end
    end
    if isequal(vFilaments, [])
        if vImarisApplication.GetFactory.IsFilaments(vDataItem)
            vFilaments = vImarisApplication.GetFactory.ToFilaments(vDataItem);
        end
    end
end

% did we find the filament?
if isequal(vFilaments, [])
    msgbox('Please create a Filaments!');
    return;
end

% did we find the spots?
if isequal(vSpots, [])
    msgbox('Please create some Spots!');
    return;
end

% ask user for a distance
vAnswer = inputdlg({ ...
  sprintf('"%s" - "%s"\nDistance to Filaments Border', ...
  char(vFilaments.GetName), char(vSpots.GetName))}, 'Find Spots center with X um close to closest Filaments Border', 1, {'1.0'});
if isempty(vAnswer)
    return;
end
vDistanceLimit = str2double(vAnswer);

%
% ALGORITHM - get spot coordinates and filament vertex coordinates from
% imaris and compute the distance between each pair. if the shortest
% distance of a spot to a filament vertex is shorter than our limit, the
% spot is close.
%

% get the spot coordinates
vSpotsPosXYZ = vSpots.GetPositionsXYZ;
vSpotsRadius = vSpots.GetRadiiXYZ;
vSpotsPosT = vSpots.GetIndicesT;
vSpotsCount = size(vSpotsPosXYZ, 1);

% bool vector for result (A is close, B is far)
vIsClose = false(vSpotsCount, 1);

vProgressDisplay = waitbar(0, 'Finding close Spots ...');


vFilamentsCount = vFilaments.GetNumberOfFilaments;
for vFilamentIndex = 0:vFilamentsCount-1
    vFilamentPosXYZ = vFilaments.GetPositionsXYZ(vFilamentIndex);
    vFilamentRadius = vFilaments.GetRadii(vFilamentIndex);
    vTimeIndex = vFilaments.GetTimeIndex(vFilamentIndex);

    % Identify Spots at the filament's same time index
    vSpotsIndex = find(vSpotsPosT == vTimeIndex);
    vSpotsCountT = numel(vSpotsIndex);
    for vIndex = 1:vSpotsCountT
        vSpotIndex = vSpotsIndex(vIndex);

        if ~vIsClose(vSpotIndex)
            % Calculate the spot distance only if no other close
            % filament had been previously found
            vDistX = vFilamentPosXYZ(:, 1) - vSpotsPosXYZ(vSpotIndex, 1);
            vDistY = vFilamentPosXYZ(:, 2) - vSpotsPosXYZ(vSpotIndex, 2);
            vDistZ = vFilamentPosXYZ(:, 3) - vSpotsPosXYZ(vSpotIndex, 3);
            vDist = sqrt(sum([vDistX, vDistY, vDistZ].^2, 2));
            %vMinRatio = min(vDist ./ vFilamentRadius);
            vLimit = min (vDist - vFilamentRadius) ;   
			%sprintf('vDist is', char(vDist), 'and vFilamentRadius is: ', char(vFilamentRadius));
            vIsClose(vSpotIndex) = vLimit < vDistanceLimit;
			%vIsClose(vSpotIndex) = lt(vLimit,vDistanceLimit);
        end
        
        waitbar((vFilamentIndex + (vIndex + 1) / vSpotsCountT) / vFilamentsCount);
    end
end
close(vProgressDisplay);

% hide original spots
vSpots.SetVisible(false);

% add new spots (close)
vSpotsA = vImarisApplication.GetFactory.CreateSpots;
% The radius can be setted to zero because it is overwritten on the next line
% Only an array with the right size is needed
vSpotsA.Set(vSpotsPosXYZ(vIsClose,:), vSpotsPosT(vIsClose), zeros(sum(vIsClose),1));
vSpotsA.SetRadiiXYZ(vSpotsRadius(vIsClose,:));
vSpotsA.SetName(sprintf('%s closer than %0.3f of border', char(vSpots.GetName), vDistanceLimit));
vSpotsA.SetColorRGBA(255);
vImarisApplication.GetSurpassScene.AddChild(vSpotsA, -1);

% add new spots (far)
vSpotsB = vImarisApplication.GetFactory.CreateSpots;
vIsFar = ~vIsClose;
vSpotsB.Set(vSpotsPosXYZ(vIsFar,:), vSpotsPosT(vIsFar),  zeros(sum(vIsFar),1));
vSpotsB.SetRadiiXYZ(vSpotsRadius(vIsFar,:));
vSpotsB.SetName(sprintf('%s farther than %0.3f of border', char(vSpots.GetName), vDistanceLimit));
vSpotsB.SetColorRGBA(255*256*256);
vImarisApplication.GetSurpassScene.AddChild(vSpotsB, -1);
